package com.chelvc.framework.security.interceptor;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.model.Result;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

/**
 * 接口权限校验拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Order(30)
@Component
public class PermissionValidateInterceptor implements Filter {
    /**
     * 权限校验
     *
     * @param request  Http请求对象
     * @param response Http相应对象
     * @return true/false
     * @throws IOException I/O操作异常
     */
    private boolean validate(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // 判断是否拥有访问权限
        SecurityProperties properties = ApplicationContextHolder.getBean(SecurityProperties.class);
        SecurityProperties.Permission config = properties.getPermission();
        if (StringUtils.isEmpty(config.getPermit()) && StringUtils.isEmpty(config.getRequire())) {
            return true;
        }
        String method = request.getMethod(), uri = HttpUtils.getRequestURI(request);
        if (StringUtils.isPath(config.getPermit(), uri) || (StringUtils.nonEmpty(config.getRequire())
                && !StringUtils.isPath(config.getRequire(), uri))
                || SecurityContextHolder.hasPermission(method, uri)) {
            return true;
        }

        // 无访问权限
        log.warn(HttpUtils.getExceptionMessage(request, HttpStatus.FORBIDDEN, "Permission denied"));
        if (StringUtils.isPath(config.getIgnore(), uri)) {
            return true;
        }
        response.setStatus(HttpStatus.FORBIDDEN.value());
        HttpUtils.writeUnifiedResponse(response, Result.build(HttpStatus.FORBIDDEN));
        return false;
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        if (this.validate((HttpServletRequest) request, (HttpServletResponse) response)) {
            chain.doFilter(request, response);
        }
    }
}
